{
  lib,
  config,
  pkgs,
  ...
}: let
  cfg = config.marleyos.xorg.i3;
  isGenericLinux = config.targets.genericLinux.enable;
in {
  options.marleyos.xorg.i3.enable = lib.mkEnableOption "i3";

  config = lib.mkIf cfg.enable {
    marleyos = {
      programs.rofi.enable = true;
      services = {
        dunst.enable = true;
        picom.enable = true;
        polybar.enable = true;
        # broken on non-nixOS
        screen-locker.enable =
          if isGenericLinux
          then false
          else true;
      };
    };

    home.packages = with pkgs; [
      networkmanagerapplet
      feh
    ];

    xsession.windowManager.i3 = {
      enable = true;
      package = lib.mkIf config.marleyos.nixGL.enable (config.lib.nixGL.wrap pkgs.i3);

      extraConfig = "tiling_drag modifier titlebar";

      config = let
        directions = {
          left = "h";
          down = "j";
          up = "k";
          right = "l";
        };
      in {
        focus.followMouse = false;
        window.border = 0;
        floating.border = 0;

        window.titlebar = false;
        floating.titlebar = false;

        gaps = {
          smartBorders = "no_gaps";
          inner = 10;
          top = 50;
        };

        bars = [];

        # Startup - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        startup = with pkgs; [
          {command = "${getExe feh} --no-fehbg --bg-fill ${./bg.jpg}";}
          {command = "${getExe networkmanagerapplet}";}
        ];

        # Keybindings - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        modifier = "Mod4";

        keybindings = let
          mod = config.xsession.windowManager.i3.config.modifier;
          workspaces = lib.range 1 10;
          getI = i:
            if i == 10
            then "0"
            else toString i;
          inherit (config.xdg) configHome;
        in
          {
            "${mod}+Shift+c" = "reload";
            "${mod}+Shift+r" = "restart";
            "${mod}+Shift+e" = "exec i3-nagbar -t warning -m 'Do you want to exit i3?' -b 'Yes' 'i3-msg exit'";

            "${mod}+e" = "layout toggle split";
            "${mod}+Control+h" = "split -";
            "${mod}+Control+v" = "split |";

            "${mod}+f" = "fullscreen toggle";

            "${mod}+Shift+space" = "floating toggle";

            "${mod}+Control+r" = "mode resize";

            "${mod}+q" = "kill";

            "${mod}+r" = "exec ${configHome}/rofi/launchers/type-1/launcher.sh";
            "${mod}+Return" = "exec ${lib.getExe config.marleyos.apps.terminal}";
          }
          # PulseAudio Volume - - - - - - - - - - - - - - - - - - - - - - - -
          // (
            lib.concatMapAttrs
            (key: cmd: {
              "${key}" = "exec --no-startup-id pactl ${cmd} && killall -SIGUSR1 i3status";
            })
            {
              XF86AudioRaiseVolume = "set-sink-volume @DEFAULT_SINK@ +10%";
              XF86AudioLowerVolume = "set-sink-volume @DEFAULT_SINK@ -10%";
              XF86AudioMute = "set-sink-mute @DEFAULT_SINK@ toggle";
              XF86AudioMicMute = "set-source-mute @DEFAULT_SOURCE@ toggle";
            }
          )
          # Focus & Movement - - - - - - - - - - - - - - - - - - - - - - - - -
          // (lib.concatMapAttrs (direction: key: {
              "${mod}+${key}" = "focus ${direction}";
              "${mod}+Shift+${key}" = "move ${direction}";
            })
            directions)
          # Switch Workspaces - - - - - - - - - - - - - - - - - - - - - - - - -
          // (lib.listToAttrs (
            (map (i: {
              name = "${mod}+${getI i}";
              value = "workspace number ${toString i}";
            }))
            workspaces
          ))
          # Move Windows Between Workspaces - - - - - - - - - - - - - - - - - -
          // (lib.listToAttrs (
            (map (i: {
              name = "${mod}+Shift+${getI i}";
              value = "move container to workspace number ${toString i}";
            }))
            workspaces
          ));

        # Resize Windows - - - - - - - - - - - - - - - - - - - - - - - - - - -
        modes.resize = let
          mod = config.xsession.windowManager.i3.config.modifier;
        in
          {
            Return = "mode default";
            Escape = "mode default";
            "${mod}+Control+r" = "mode default";
          }
          // (
            lib.concatMapAttrs
            (key: action: {
              "${key}" = "resize ${action} 10 px or 10 ppt";
            })
            rec {
              Left = "shrink width";
              Down = "shrink height";
              Up = "grow height";
              Right = "grow width";
              "${directions.left}" = Left;
              "${directions.down}" = Down;
              "${directions.up}" = Up;
              "${directions.right}" = Right;
            }
          );
      };
    };
  };
}
